seconds_per_minute = 60
minutes_per_hour = 60
hours_per_day = 24
milliseconds_per_second = 1000
seconds_per_hour = seconds_per_minute * minutes_per_hour
seconds_per_day = seconds_per_hour * hours_per_day
microseconds_per_second = 1000 * milliseconds_per_second
